<?php
class Rontar_DynamicRetargeting_IndexController extends Mage_Core_Controller_Front_Action 
{
	public function indexAction() 
	{
		header('Content-type: text/xml');
		
		set_time_limit(7200);	

		$this->WriteXml();
	}
	
	public function WriteXml()
	{
		$helper = new Rontar_DynamicRetargeting_Helper_Data();
	
		$xmlWriter = new XMLWriter();
		$xmlWriter->openURI('php://output');
		$xmlWriter->setIndent(true);
		$xmlWriter->startDocument('1.0', 'UTF-8');
		
		$xmlWriter->startElement('shop');
		
		// currencies
		$xmlWriter->startElement('currencies');
		$xmlWriter->startElement('currency');		
		$xmlWriter->writeAttribute('id', Mage::app()->getStore()->getCurrentCurrencyCode());
		$xmlWriter->writeAttribute('rate', '1');		
		$xmlWriter->endElement(); // currency
		$xmlWriter->endElement(); // currencies
		
		// categories
		$xmlWriter->startElement('categories');		
		
		$CategoryIds = Mage::getModel('catalog/category')->getCollection()->getAllIds();
		foreach ($CategoryIds as $CategoryId)
		{
			$Category = Mage::getModel('catalog/category');
			$Category->load($CategoryId);
			
			if ($Category->getIsActive()) 
			{
				$xmlWriter->startElement('category');
				$xmlWriter->writeAttribute('id', $Category->getId());
				$xmlWriter->text($Category->getName());
				$xmlWriter->endElement(); // category
			}
		}
		$xmlWriter->endElement(); // categories
		// offers
		$xmlWriter->startElement('offers');
		$ProductsIds = Mage::getModel('catalog/product')->getCollection()->getAllIds();

		foreach ($ProductsIds as $ProductId)
		{
			$Product = Mage::getSingleton('catalog/product')->load($ProductId);
	      	if ($Product->getStatus() != Mage_Catalog_Model_Product_Status::STATUS_ENABLED)
				continue;
		        
		    $stock = Mage::getSingleton('cataloginventory/stock_item')->loadByProduct($Product);
		    if (!$stock->getIsInStock()) continue;
            $stock->clearInstance();
            unset($stock);

                        $parentIds = Mage::getModel('catalog/product_type_configurable')->getParentIdsByChild($ProductId);
    		        if ($parentIds != null) continue;
				
			$xmlWriter->startElement('offer');
			$xmlWriter->writeAttribute('id', $Product->getId());
			$xmlWriter->startElement('name');
			$xmlWriter->writeCdata($Product->getName());
			$xmlWriter->endElement(); // name
			$xmlWriter->writeElement('url', $Product->getProductUrl());
			$xmlWriter->writeElement('price', Mage::helper('tax')->getPrice($Product, $Product->getPrice(), true, null, null, null, null, false));
			$xmlWriter->writeElement('picture', $Product->getImageUrl());
			
			$categoryId = $Product->getCategoryIds();
			if ($categoryId <> null)
				$xmlWriter->writeElement('categoryId', $categoryId[0]);
			
			$xmlWriter->endElement(); // offer

            $Product->clearInstance();
            unset($Product);
		}

		$xmlWriter->endElement(); // offers
		$xmlWriter->endElement(); // shop
		
		$xmlWriter->endDocument();	
		$xmlWriter->flush();
 	}
	
	
	public function CreateXml()
	{
		$helper = new Rontar_DynamicRetargeting_Helper_Data();
		
		$xml_feed = new Rontar_DynamicRetargeting_SimpleXml('<?xml version="1.0" encoding="utf-8"?><shop></shop>');
		
		// currencies
		$xmlCurrencies = $xml_feed->addChild('currencies');	
		$xmlCurrency = $xmlCurrencies->addChild('currency');
		$xmlCurrency->addAttribute("id", Mage::app()->getStore()->getCurrentCurrencyCode());
		$xmlCurrency->addAttribute("rate", "1");
		
		// categories
		$xmlCategories = $xml_feed->addChild('categories');	
		$CategoryIds = Mage::getModel('catalog/category')->getCollection()->getAllIds();

		foreach ($CategoryIds as $CategoryId)
		{
			$Category = Mage::getModel('catalog/category');
			$Category->load($CategoryId);
			
			if ($Category->getIsActive()) 
			{
				$xmlCategory = $xmlCategories->addChild('category', $Category->getName());
				$xmlCategory->addAttribute("id", $Category->getId());
			}
		}
		
		// offers
		$xmlOffers = $xml_feed->addChild('offers');
		
		$ProductsIds = Mage::getModel('catalog/product')->getCollection()->getAllIds();
		$Products = array();
		foreach ($ProductsIds as $ProductId)
		{
			$Product = Mage::getModel('catalog/product')->load($ProductId);
			array_push($Products, $Product);
		}		
		
		foreach ($Products as $Product)
		{
			if (!$helper->IsProductAvailable($Product))
				continue;
				
			$xmlOffer = $xmlOffers->addChild('offer');
			$xmlOffer->addAttribute("id", $Product->getId());
			
			$xmlOffer->addChild("name", $Product->getName());
			$xmlOffer->addChild("url", $Product->getProductUrl());
			$xmlOffer->addChild("price", $Product->getFinalPrice());
			$xmlOffer->addChild("picture", $Product->getImageUrl());
			
			$categoryId = $Product->getCategoryIds();
			if ($categoryId <> null)
				$xmlOffer->addChild("categoryId", $categoryId[0]);	
		}
		
		return $xml_feed->GetXML();
	}
}

class Rontar_DynamicRetargeting_SimpleXml extends SimpleXMLElement
{
	public function GetXML()
	{
		$xml_string = $this->asXML();
		$dom = new DOMDocument('1.0', 'UTF-8');
		$dom->loadXML($xml_string);
		$dom->preserveWhiteSpace = FALSE;
		$dom->formatOutput = TRUE;
		return $dom->saveXML();
	}
}
